--[[
Copyright (C) GtX (Andy), 2020

Author: GtX | Andy
Date: 24.08.2020
Revision: FS22-01

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Important:
Not to be added to any mods / maps or modified from its current release form.
No modifications may be made to this script, including conversion to other game versions without written permission from GtX | Andy

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
Ohne schriftliche Genehmigung von GtX | Andy dürfen keine Änderungen an diesem Skript vorgenommen werden, einschließlich der Konvertierung in andere Spielversionen
]]

local vehicleSpeedSync
local validationFail
local firstTimeLoad

local buildId = 1
local versionString = "0.0.0.0"

local modName = g_currentModName or ""
local modDirectory = g_currentModDirectory or ""
local modSettingsDirectory = g_currentModSettingsDirectory or ""

local colourEnabled = true

local standardColour = {
    0.0953,
    1,
    0.0685,
    1
}

local colourBlindColour = {
    1,
    0,
    1,
    1
}

local syncedColour = {
    0.0953,
    1,
    0.0685,
    1
}

local function getSyncedColour(vehicle, missionTime)
    if vehicle:getSyncedSpeedWaiting() and math.min(math.max(math.cos(8.0 * (missionTime / 1000)) + 0.2, 0), 1) == 0 then
        return 1, 1, 1, 1
    end

    return syncedColour[1], syncedColour[2], syncedColour[3], syncedColour[4]
end

local function typeManager_validateTypes(typeManager)
    if typeManager.typeName == "vehicle" then
        local specializationName = string.format("%s.vehicleSpeedSync", modName)
        local specializationObject = g_specializationManager:getSpecializationObjectByName(specializationName)

        if specializationObject ~= nil then
            for typeName, typeEntry in pairs (typeManager:getTypes()) do
                if specializationObject.prerequisitesPresent(typeEntry.specializations) then
                    typeManager:addSpecialization(typeName, specializationName)

                    vehicleSpeedSync.numInserted = vehicleSpeedSync.numInserted + 1
                end
            end
        end
    end
end

local function fsBaseMission_loadMapFinished(mission)
    if (vehicleSpeedSync ~= nil and vehicleSpeedSync.numInserted > 0) and mission.hud ~= nil then
        if colourEnabled and mission.hud.speedMeter ~= nil then
            local speedMeter = mission.hud.speedMeter

            speedMeter.updateSpeedGauge = Utils.overwrittenFunction(speedMeter.updateSpeedGauge, function(speedMeterDisplay, superFunc, dt)
                superFunc(speedMeterDisplay, dt)

                if speedMeterDisplay.vehicle.getSyncedSpeedActive ~= nil and speedMeterDisplay.vehicle:getSyncedSpeedActive() then
                    if speedMeterDisplay.originalGaugeColour == nil then
                        speedMeterDisplay.originalGaugeColour = {speedMeterDisplay.gaugeBackgroundElement:getColor()}
                    end

                    speedMeterDisplay.gaugeBackgroundElement:setColor(getSyncedColour(speedMeterDisplay.vehicle, g_currentMission.time))
                elseif speedMeterDisplay.originalGaugeColour ~= nil then
                    speedMeterDisplay.gaugeBackgroundElement:setColor(unpack(speedMeterDisplay.originalGaugeColour))
                    speedMeterDisplay.originalGaugeColour = nil
                end
            end)

            if vehicleSpeedSync.onColourBlindModeChanged ~= nil then
                local useColorBlindMode = false

                if GameSettings.SETTING.USE_COLORBLIND_MODE ~= nil then
                    g_messageCenter:subscribe(MessageType.SETTING_CHANGED[GameSettings.SETTING.USE_COLORBLIND_MODE], vehicleSpeedSync.onColourBlindModeChanged, vehicleSpeedSync)
                    mission:addNonUpdateable(vehicleSpeedSync)

                    useColorBlindMode = g_gameSettings:getValue(GameSettings.SETTING.USE_COLORBLIND_MODE)
                end

                vehicleSpeedSync.onColourBlindModeChanged(vehicleSpeedSync, useColorBlindMode)
            end
        end

        -- Guidance Steering
        local environment = _G["FS22_guidanceSteering"]

        if environment == nil and (mission.guidanceSteering ~= nil and mission.guidanceSteering.modName ~= nil) then
            environment = _G[mission.guidanceSteering.modName]
        end

        if environment ~= nil and environment.DriveUtil ~= nil then
            environment.DriveUtil.accelerateInDirection = Utils.overwrittenFunction(environment.DriveUtil.accelerateInDirection, function(vehicle, superFunc, axisForward, dt, forceBrake)
                local spec = vehicle.spec_vehicleSpeedSync

                if spec ~= nil and spec.speedSyncActive then
                    if axisForward == 0 then
                        if vehicle:getVehicleIsInRange(spec.vehicleInRange, spec.setRange) then
                            local motor = vehicle:getMotor()

                            local speedLimit = motor:getSpeedLimit(true)
                            local lastSpeed = spec.vehicleInRange:getLastSpeed()

                            if lastSpeed > 0.5 and lastSpeed < speedLimit + 5 then
                                axisForward = MathUtil.sign(spec.vehicleInRange.movingDirection or 1)
                                motor:setSpeedLimit(math.min(lastSpeed, speedLimit))
                                spec.isWaiting = false
                            elseif not spec.isWaiting then
                                vehicle:setSyncedSpeedState(false, nil, true)
                            end
                        else
                            vehicle:setSyncedSpeedState(false, nil, true)
                        end
                    else
                        vehicle:setSyncedSpeedState(false)
                    end
                end

                return superFunc(vehicle, axisForward, dt, forceBrake)
            end)
        end
    end
end

local function loadSettingsFromXML()
    if modSettingsDirectory == nil or modSettingsDirectory == "" then
        modSettingsDirectory = getUserProfileAppPath() .. "modSettings/" .. modName

        if modSettingsDirectory == nil or modSettingsDirectory == "" then
            return true
        end
    end

    local filepath = modSettingsDirectory .. "settings.xml"

    if fileExists(filepath) then
        local xmlFile = loadXMLFile("VehicleSpeedSyncXML", filepath)

        if xmlFile ~= nil and xmlFile ~= 0 then
            VehicleSpeedSync.SPEED_SYNC_VISIBLE = Utils.getNoNil(getXMLBool(xmlFile, "vehicleSpeedSync.activeSpeedSync#showInputBinding"), true)
            VehicleSpeedSync.SPEED_SYNC_ACTIVE = Utils.getNoNil(getXMLBool(xmlFile, "vehicleSpeedSync.activeSpeedSync#enabled"), true)

            colourEnabled = Utils.getNoNil(getXMLBool(xmlFile, "vehicleSpeedSync.activeSpeedSync.gaugeColour#enabled"), colourEnabled)

            if colourEnabled then
                standardColour = Utils.getNoNil(string.getVectorN(getXMLString(xmlFile, "vehicleSpeedSync.activeSpeedSync.gaugeColour#standard"), 4), standardColour)

                if standardColour[4] <= 0 then
                    standardColour[4] = 1
                end

                colourBlindColour = Utils.getNoNil(string.getVectorN(getXMLString(xmlFile, "vehicleSpeedSync.activeSpeedSync.gaugeColour#colourBlindMode"), 4), colourBlindColour)

                if colourBlindColour[4] <= 0 then
                    colourBlindColour[4] = 1
                end
            end

            VehicleSpeedSync.CRUISE_SYNC_VISIBLE = Utils.getNoNil(getXMLBool(xmlFile, "vehicleSpeedSync.cruiseControlSync#showInputBinding"), true)
            VehicleSpeedSync.CRUISE_SYNC_ACTIVE = Utils.getNoNil(getXMLBool(xmlFile, "vehicleSpeedSync.cruiseControlSync#enabled"), true)

            delete(xmlFile)
        end

        return false, filepath
    else
        createFolder(modSettingsDirectory)
        copyFile(modDirectory .. "resources/settings.xml", filepath, false)
    end

    return true, filepath
end

local function init()
    if g_globalMods and g_globalMods.vehicleSpeedSync then
        Logging.error("Failed to load mod '%s', 'VehicleSpeedSync.lua' was already loaded by '%s'!", modName, g_globalMods.vehicleSpeedSync.modName or "N/A")

        return
    end

    local isValidMod = false
    local mod = g_modManager:getModByName(modName)

    versionString = mod.version or versionString

    if mod.modName == "FS22_VehicleSpeedSync" or mod.modName == "FS22_VehicleSpeedSync_update" then
        isValidMod = mod.author and #mod.author == 3
    end

    if isValidMod then
        vehicleSpeedSync = {
            numInserted = 0,
            modName = modName,
            version = version,
            buildId = buildId,
            colourEnabled = colourEnabled,
            onColourBlindModeChanged = function(self, useColorBlindMode)
                syncedColour = useColorBlindMode and colourBlindColour or standardColour
            end,
            delete = function(self)
                g_messageCenter:unsubscribeAll(self)
            end
        }

        if g_globalMods then
            g_globalMods.vehicleSpeedSync = vehicleSpeedSync
        end

        source(modDirectory .. "scripts/specializations/events/VehicleSpeedSyncEvent.lua")
        g_specializationManager:addSpecialization("vehicleSpeedSync", "VehicleSpeedSync", modDirectory .. "scripts/specializations/VehicleSpeedSync.lua", nil)

        FSBaseMission.loadMapFinished = Utils.prependedFunction(FSBaseMission.loadMapFinished, fsBaseMission_loadMapFinished)
        TypeManager.validateTypes = Utils.prependedFunction(TypeManager.validateTypes, typeManager_validateTypes)

        if g_dedicatedServerInfo == nil then
            local isFirstLoad, filepath = loadSettingsFromXML()

            if isFirstLoad and filepath ~= nil then
                firstTimeLoad = {
                    startUpdateTime = 2000,

                    update = function(self, dt)
                        self.startUpdateTime = self.startUpdateTime - dt

                        if self.startUpdateTime < 0 and not g_gui:getIsGuiVisible() then
                            local message = string.format(g_i18n:getText("vehicleSpeedSync_firstTimeRun", modName), filepath:gsub(" ", ""))

                            g_currentMission.hud:showInGameMessage(string.format("%s - Version %s", mod.title, versionString), message, -1)

                            removeModEventListener(firstTimeLoad)
                            firstTimeLoad = nil
                        end
                    end
                }

                addModEventListener(firstTimeLoad)
            end
        end
    else
        validationFail = {
            startUpdateTime = 2000,

            update = function(self, dt)
                self.startUpdateTime = self.startUpdateTime - dt

                if self.startUpdateTime < 0 then
                    local text = string.format(g_i18n:getText("vehicleSpeedSync_loadError", modName), modName, mod.author or "Unknown")

                    if g_dedicatedServerInfo == nil then
                        if not g_gui:getIsGuiVisible() then
                            g_gui:showYesNoDialog({
                                title = string.format("%s - Version %s", mod.title, versionString),
                                text = text,
                                dialogType = DialogElement.TYPE_LOADING,
                                callback = self.openModHubLink,
                                yesText = g_i18n:getText("button_ok"),
                                noText = g_i18n:getText("button_modHubDownload")
                            })
                        end
                    else
                        print("\n" .. text .. "\n    - https://farming-simulator.com/mods.php?lang=en&country=be&title=fs2022&filter=org&org_id=129652&page=0" .. "\n")
                        self.openModHubLink(true)
                    end
                end
            end,

            openModHubLink = function(ignore)
                if ignore == false then
                    local language = g_languageShort
                    local link = "mods.php?lang=en&country=be&title=fs2022&filter=org&org_id=129652&page=0"
                    if language == "de" or language == "fr" then
                        link = "mods.php?lang=" .. language .. "&country=be&title=fs2022&filter=org&org_id=129652&page=0"
                    end

                    openWebFile(link, "")
                end

                removeModEventListener(validationFail)
                validationFail = nil
            end
        }

        addModEventListener(validationFail)
    end
end

init()
